@@ -59,7 +59,8 @@
 #define SETPENCOLOR    6
 #define SETPENSIZE     7
 #define SETPENPATTERN  8
-#define FILLERUP	   9
+#define FILLERUP       9
+#define ARC	      10
 
 /* NOTE: See the files (macterm.c and macterm.h) or (ibmterm.c and ibmterm.h)
    for examples of the functions and macros that this file assumes exist. */
@@ -67,6 +68,7 @@
 #define One (sizeof(int))
 #define Two (2*One)
 #define Three (3*One)
+#define Big (sizeof(FLONUM))
 
 #define PENMODE_PAINT	0
 #define PENMODE_ERASE	1
@@ -82,11 +84,13 @@ FLONUM wanna_x = 0.0, wanna_y = 0.0;
 BOOLEAN out_of_bounds = FALSE;
 void setpos_bynumber(FLONUM, FLONUM);
 
+int max_palette_slot = 0;
+
 char record[GR_SIZE];
-int record_index = 0;
+FIXNUM record_index = 0;
 pen_info orig_pen;
 
-BOOLEAN record_next_move = FALSE, refresh_p = TRUE;
+BOOLEAN refresh_p = TRUE;
 
 /************************************************************/
 
@@ -158,12 +162,15 @@ void check_x_low(void) {
 	turtle_x -= (screen_width + 1);
     }
 }
-
 void draw_turtle_helper(void) {
     pen_info saved_pen;
     FLONUM real_heading;
     int left_x, left_y, right_x, right_y, top_x, top_y;
-    
+#if 1	/* Evan Marshall Manning <manning@alumni.caltech.edu> */
+    double cos_real_heading, sin_real_heading;
+    FLONUM delta_x, delta_y;
+#endif
+   
     prepare_to_draw;
     prepare_to_draw_turtle;
     save_pen(&saved_pen);
@@ -171,25 +178,51 @@ void draw_turtle_helper(void) {
     pen_vis = 0;
     set_pen_width(1);
     set_pen_height(1);
-    
+   
     real_heading = -turtle_heading + 90.0;
-	
+ 
+#if 0
     left_x = g_round(turtle_x + x_scale*(FLONUM)(cos((FLONUM)((real_heading + 90.0)*degrad))*turtle_half_bottom));
     left_y = g_round(turtle_y + y_scale*(FLONUM)(sin((FLONUM)((real_heading + 90.0)*degrad))*turtle_half_bottom));
-
+ 
     right_x = g_round(turtle_x + x_scale*(FLONUM)(cos((FLONUM)((real_heading - 90.0)*degrad))*turtle_half_bottom));
     right_y = g_round(turtle_y + y_scale*(FLONUM)(sin((FLONUM)((real_heading - 90.0)*degrad))*turtle_half_bottom));
-
+ 
     top_x = g_round(turtle_x + x_scale*(FLONUM)(cos((FLONUM)(real_heading*degrad))*turtle_side));
     top_y = g_round(turtle_y + y_scale*(FLONUM)(sin((FLONUM)(real_heading*degrad))*turtle_side));
-
+#else
+    cos_real_heading = cos((FLONUM)(real_heading*degrad));
+    sin_real_heading = sin((FLONUM)(real_heading*degrad));
+ 
+    delta_x = x_scale*(FLONUM)(sin_real_heading*turtle_half_bottom);
+    delta_y = y_scale*(FLONUM)(cos_real_heading*turtle_half_bottom);
+ 
+    left_x = g_round(turtle_x - delta_x);
+    left_y = g_round(turtle_y + delta_y);
+ 
+    right_x = g_round(turtle_x + delta_x);
+    right_y = g_round(turtle_y - delta_y);
+ 
+    top_x = g_round(turtle_x + x_scale*(FLONUM)(cos_real_heading*turtle_side));
+    top_y = g_round(turtle_y + y_scale*(FLONUM)(sin_real_heading*turtle_side));
+#endif
+ 
+#if 0
+    /* move to left, draw to top; move to right, draw to top; move to left draw to right */
     move_to(screen_x_center + left_x, screen_y_center - left_y);
     line_to(screen_x_center + top_x, screen_y_center - top_y);
     move_to(screen_x_center + right_x, screen_y_center - right_y);
     line_to(screen_x_center + top_x, screen_y_center - top_y);
     move_to(screen_x_center + left_x, screen_y_center - left_y);
     line_to(screen_x_center + right_x, screen_y_center - right_y);
-
+#else
+    /* move to right, draw to left, draw to top, draw to right */
+    move_to(screen_x_center + right_x, screen_y_center - right_y);
+    line_to(screen_x_center + left_x, screen_y_center - left_y);
+    line_to(screen_x_center + top_x, screen_y_center - top_y);
+    line_to(screen_x_center + right_x, screen_y_center - right_y);
+#endif
+ 
     restore_pen(&saved_pen);
     done_drawing_turtle;
     done_drawing;
@@ -201,6 +234,7 @@ BOOLEAN safe_to_save(void);
 void save_line(void), save_move(void), save_vis(void), save_mode(void);
 void save_color(void), save_size(void), save_pattern(void);
 void save_string(char *, int);
+void save_arc(FLONUM, FLONUM, FLONUM, FLONUM, FLONUM, FLONUM, FLONUM, FLONUM);
 
 void right(FLONUM a) {
     draw_turtle();
@@ -270,6 +304,7 @@ void forward(FLONUM d) {
 
 void forward_helper(FLONUM d) {
     FLONUM real_heading, dx, dy, x1, y1, x2, y2, newd = 0.0;
+    FIXNUM rx2, ry2;
 
 wraploop:
     if (newd != 0.0) d = newd;
@@ -284,10 +319,7 @@ wraploop:
     y2 = y1 - dy;
     
     move_to(g_round(x1), g_round(y1));
-    if (record_next_move) {
-	save_move();
-	record_next_move = FALSE;
-    }
+    save_move();
     
     if (check_throwing) return;
 
@@ -296,12 +328,15 @@ wraploop:
 	save_line();
     }
 
+    rx2 = g_round(x2);
+    ry2 = g_round(y2);
+
     if (current_mode == windowmode ||
-	(x2 >= screen_left && x2 <= screen_right &&
-	 y2 >= screen_top && y2 <= screen_bottom)) {
+	(rx2 >= screen_left && rx2 <= screen_right &&
+	 ry2 >= screen_top && ry2 <= screen_bottom)) {
 	turtle_x = turtle_x + dx;
 	turtle_y = turtle_y + dy;
-	line_to(g_round(x2), g_round(y2));
+	line_to(rx2, ry2);
 	save_line();
     } else {
 	if (newd = wrap_right(d, x1, y1, x2, y2)) goto wraploop;
@@ -318,13 +353,14 @@ wraploop:
 
 FLONUM wrap_right(FLONUM d, FLONUM x1, FLONUM y1, FLONUM x2, FLONUM y2) {
     FLONUM yi, newd;
+    FIXNUM ryi;
     
     if (x2 > screen_right) {
 	yi = ((y2 - y1)/(x2 - x1)) * (screen_right + 1 - x1) + y1;
-	if (yi >= screen_top && yi <= screen_bottom) {
-	    line_to(screen_right, g_round(yi));
+	ryi = g_round(yi);
+	if (ryi >= screen_top && ryi <= screen_bottom) {
+	    line_to(screen_right, ryi);
 	    save_line();
-	    record_next_move = TRUE;
 	    turtle_x = turtle_left_max;
 	    turtle_y = screen_y_center - yi;
 	    if (current_mode == wrapmode) {
@@ -341,13 +377,14 @@ FLONUM wrap_right(FLONUM d, FLONUM x1, FLONUM y1, FLONUM x2, FLONUM y2) {
 
 FLONUM wrap_left(FLONUM d, FLONUM x1, FLONUM y1, FLONUM x2, FLONUM y2) {
     FLONUM yi, newd;
+    FIXNUM ryi;
     
     if (x2 < screen_left) {
 	yi = ((y1 - y2)/(x2 - x1)) * (x1 + 1 - screen_left) + y1;
-	if (yi >= screen_top && yi <= screen_bottom) {
-	    line_to(screen_left, g_round(yi));
+	ryi = g_round(yi);
+	if (ryi >= screen_top && ryi <= screen_bottom) {
+	    line_to(screen_left, ryi);
 	    save_line();
-	    record_next_move = TRUE;
 	    turtle_x = turtle_right_max;
 	    turtle_y = screen_y_center - yi;
 	    if (current_mode == wrapmode) {
@@ -364,13 +401,27 @@ FLONUM wrap_left(FLONUM d, FLONUM x1, FLONUM y1, FLONUM x2, FLONUM y2) {
 
 FLONUM wrap_up(FLONUM d, FLONUM x1, FLONUM y1, FLONUM x2, FLONUM y2) {
     FLONUM xi, newd;
-    
+    FIXNUM rxi;
+
     if (y2 < screen_top) {
 	xi = ((x2 - x1)/(y1 - y2)) * (y1 + 1 - screen_top) + x1;
-	if (xi >= screen_left && xi <= screen_right) {
-	    line_to(g_round(xi), screen_top);
+	rxi = g_round(xi);
+	if (rxi >= screen_left && rxi <= screen_right) {
+	    line_to(rxi, screen_top);
+	    save_line();
+	    turtle_x = xi - screen_x_center;
+	    turtle_y = turtle_bottom_max;
+	    if (current_mode == wrapmode) {
+		newd = d * ((y2 + 1 - screen_top)/(y2 - y1));
+		if (newd*d > 0) return(newd);
+	    } else {
+		turtle_y = turtle_top_max;
+		err_logo(TURTLE_OUT_OF_BOUNDS, NIL);
+	    }
+	} else if (rxi >= (screen_left-1) && rxi <= (screen_right+1)) {
+	    rxi = (rxi > screen_right ? screen_right : screen_left);
+	    line_to(rxi, screen_top);
 	    save_line();
-	    record_next_move = TRUE;
 	    turtle_x = xi - screen_x_center;
 	    turtle_y = turtle_bottom_max;
 	    if (current_mode == wrapmode) {
@@ -387,13 +438,27 @@ FLONUM wrap_up(FLONUM d, FLONUM x1, FLONUM y1, FLONUM x2, FLONUM y2) {
 
 FLONUM wrap_down(FLONUM d, FLONUM x1, FLONUM y1, FLONUM x2, FLONUM y2) {
     FLONUM xi, newd;
+    FIXNUM rxi;
     
     if (y2 > screen_bottom) {
 	xi = ((x2 - x1)/(y2 - y1)) * (screen_bottom + 1 - y1) + x1;
-	if (xi >= screen_left && xi <= screen_right) {
-	    line_to(g_round(xi), screen_bottom);
+	rxi = g_round(xi);
+	if (rxi >= screen_left && rxi <= screen_right) {
+	    line_to(rxi, screen_bottom);
+	    save_line();
+	    turtle_x = xi - screen_x_center;
+	    turtle_y = turtle_top_max;
+	    if (current_mode == wrapmode) {
+		newd = d * ((y2 - screen_bottom - 1)/(y2 - y1));
+		if (newd*d > 0) return(newd);
+	    } else {
+		turtle_y = turtle_bottom_max;
+		err_logo(TURTLE_OUT_OF_BOUNDS, NIL);
+	    }
+	} else if (rxi >= (screen_left-1) && rxi <= (screen_right+1)) {
+	    rxi = (rxi > screen_right ? screen_right : screen_left);
+	    line_to(rxi, screen_bottom);
 	    save_line();
-	    record_next_move = TRUE;
 	    turtle_x = xi - screen_x_center;
 	    turtle_y = turtle_top_max;
 	    if (current_mode == wrapmode) {
@@ -590,6 +655,11 @@ void cs_helper(int centerp) {
     p_info_x(orig_pen) = g_round(screen_x_coord);
     p_info_y(orig_pen) = g_round(screen_y_coord);
     record_index = 0;
+    if (turtle_x != 0.0 || turtle_y != 0.0) save_move();
+    if (pen_vis != 0) save_vis();
+    if (internal_penmode != PENMODE_PAINT) save_mode();
+    if (pen_color != 7) save_color();
+    if (pen_width != 1 || pen_height != 1) save_size();
     done_drawing;
 }
 
@@ -772,7 +842,6 @@ NODE *llabel(NODE *arg) {
 #endif
 	label(textbuf);
 	save_string(textbuf,theLength);
-	record_next_move = TRUE;
 	draw_turtle();
 	done_drawing;
     }
@@ -879,14 +948,16 @@ NODE *lsetbackground(NODE *arg) {
 NODE *lsetpalette(NODE *args) {
 	NODE *slot = pos_int_arg(args);
 	NODE *arg = rgb_arg(cdr(args));
+	int slotnum = (int)getint(slot);
 
-	if (NOT_THROWING && ((int)getint(slot) > 7)) {
+	if (NOT_THROWING && (slotnum > 7)) {
 		prepare_to_draw;
-		set_palette((int)getint(slot),
+		set_palette(slotnum,
 			    (unsigned int)getint(car(arg)),
 			    (unsigned int)getint(cadr(arg)),
 			    (unsigned int)getint(car(cddr(arg))));
 		done_drawing;
+		if (slotnum > max_palette_slot) max_palette_slot = slotnum;
 	}
 	return(UNBOUND);
 }
@@ -904,6 +975,27 @@ NODE *lpalette(NODE *args) {
 	return UNBOUND;
 }
 
+void save_palette(FILE *fp) {
+    unsigned int colors[3];
+    int i;
+    fwrite(&max_palette_slot, sizeof(int), 1, fp);
+    for (i=8; i <= max_palette_slot; i++) {
+	get_palette(i, &colors[0], &colors[1], &colors[2]);
+	fwrite(colors, sizeof(int), 3, fp);
+    }
+}
+
+void restore_palette(FILE *fp) {
+    unsigned int colors[3];
+    int i, nslots;
+    fread(&nslots, sizeof(int), 1, fp);
+    if (nslots > max_palette_slot) max_palette_slot = nslots;
+    for (i=8; i <= nslots; i++) {
+	fread(colors, sizeof(int), 3, fp);
+	set_palette(i, colors[0], colors[1], colors[2]);
+    }
+}
+
 NODE *lsetpensize(NODE *args) {
     NODE *arg = pos_int_vector_arg(args);
 
@@ -982,7 +1074,6 @@ NODE *lbuttonp(NODE *args) {
 }
 
 NODE *ltone(NODE *args) {
-#ifndef WIN32
     NODE *p, *d;
     FIXNUM pitch, duration;
     
@@ -994,13 +1085,57 @@ NODE *ltone(NODE *args) {
 	duration = (nodetype(d) == FLOATT) ? (FIXNUM)getfloat(d) : getint(d);
 	if (pitch > 0) tone(pitch, duration);
     }
-#else
-    MessageBeep(0xffffffff);
-#endif
-    
     return(UNBOUND);
 }
 
+void do_arc(FLONUM count, FLONUM ang, FLONUM radius, FLONUM delta,
+	    FLONUM tx, FLONUM ty, FLONUM angle, FLONUM thead, BOOLEAN save) {
+    FLONUM x;
+    FLONUM y;
+    FLONUM i;
+    BOOLEAN save_refresh = refresh_p;
+    FLONUM save_x = turtle_x;
+    FLONUM save_y = turtle_y;
+    int pen_state;
+
+    refresh_p = 0;
+    if (save) {
+	x = sin(ang*3.141592654/180.0)*radius;
+	y = cos(ang*3.141592654/180.0)*radius;
+	turtle_x = tx+x;
+	turtle_y = ty+y;
+    }
+
+    /* draw each line segment of arc (will do wrap) */
+
+    for (i=1.0;i<=count;i=i+1.0) {
+
+       /* calc x y */
+
+       x = sin(ang*3.141592654/180.0)*radius;
+       y = cos(ang*3.141592654/180.0)*radius;
+       setpos_bynumber(tx+x, ty+y);
+       ang = ang + delta;
+    }
+
+    /* assure we draw something and end in the exact right place */
+
+    x = sin((thead+angle)*3.141592654/180.0)*radius;
+    y = cos((thead+angle)*3.141592654/180.0)*radius;
+
+    setpos_bynumber(tx+x, ty+y);
+
+    if (save) {
+	pen_state = pen_vis;
+	pen_vis = -1;
+	setpos_bynumber(tx, ty);
+	pen_vis = pen_state;
+	turtle_x = save_x;
+	turtle_y = save_y;
+    }
+    refresh_p = save_refresh;
+}
+
 NODE *larc(NODE *arg) {
     NODE *val1;
     NODE *val2;
@@ -1010,11 +1145,10 @@ NODE *larc(NODE *arg) {
     FLONUM ang;
     FLONUM tx;
     FLONUM ty;
-    FLONUM x;
-    FLONUM y;
     FLONUM count;
     FLONUM delta;
-    FLONUM i;
+    FLONUM x;
+    FLONUM y;
 
     int turtle_state;
     int pen_state;
@@ -1036,84 +1170,65 @@ NODE *larc(NODE *arg) {
 	else
 	    radius = getfloat(val2);
 
-    prepare_to_draw;
-    draw_turtle();
+	prepare_to_draw;
+	draw_turtle();
 
-    /* save and force turtle state */
+	/* save and force turtle state */
 
-    turtle_state = turtle_shown;
-    turtle_shown = 0;
+	turtle_state = turtle_shown;
+	turtle_shown = 0;
 
-    /* grab things before they change and use for restore */
+	/* grab things before they change and use for restore */
 
-    ang = turtle_heading;
-    tx = turtle_x;
-    ty = turtle_y;
+	ang = turtle_heading;
+	tx = turtle_x;
+	ty = turtle_y;
 
-    /* calculate resolution parameters */
+	/* calculate resolution parameters */
 
-    count = abs(angle*radius/200.0);
-    if (count == 0.0) count = 1.0;
-    delta = angle/count;
+	count = fabs(angle*radius/200.0);	/* 4.5 */
+	if (count == 0.0) count = 1.0;
+	delta = angle/count;
 
-    /* draw each line segment of arc (will do wrap) */
+	/* jump to begin of first line segment without drawing */
 
-    for (i=0.0;i<=count;i=i+1.0)
-       {
+	x = sin(ang*3.141592654/180.0)*radius;
+	y = cos(ang*3.141592654/180.0)*radius;
 
-       /* calc x y */
+	pen_state = pen_vis;
+	pen_vis = -1;
+	save_vis();
+	setpos_bynumber(tx+x, ty+y);
+	pen_vis = pen_state;
+	save_vis();
+	ang = ang + delta;
 
-       x = sin(ang*3.141592654/180.0)*radius;
-       y = cos(ang*3.141592654/180.0)*radius;
+	save_arc(count, ang, radius, delta, tx, ty, angle, turtle_heading);
 
-       /* jump to begin of first line segment without drawing */
+	do_arc(count, ang, radius, delta, tx, ty, angle, turtle_heading, 0);
 
-       if (i==0.0)
-          {
-          pen_state = pen_vis;
-          pen_vis = -1;
-	  save_vis();
-          setpos_bynumber(tx+x, ty+y);
-          pen_vis = pen_state;
-	  save_vis();
-          }
+	/* restore state */
 
-       /* else do segment */
+	pen_state = pen_vis;
+	pen_vis = -1;
+	save_vis();
+	setpos_bynumber(tx, ty);
+	pen_vis = pen_state;
+	save_vis();
 
-       else
-          {
-          setpos_bynumber(tx+x, ty+y);
-          }
+	turtle_shown = turtle_state;
+	draw_turtle();
+	wanna_x = turtle_x;
+	wanna_y = turtle_y;
+	out_of_bounds = FALSE;
+	pen_state = pen_vis;
+	pen_vis = -1;
+	save_vis();
+	forward_helper((FLONUM)0.0);    /* Lets fill work -- dunno why */
+	pen_vis = pen_state;
+	save_vis();
 
-       ang = ang + delta;
-       }
-
-    /* assure we draw something and end in the exact right place */
-
-    x = sin((turtle_heading+angle)*3.141592654/180.0)*radius;
-    y = cos((turtle_heading+angle)*3.141592654/180.0)*radius;
-
-    setpos_bynumber(tx+x, ty+y);
-
-    /* restore state */
-
-    turtle_shown = turtle_state;
-
-    turtle_x = tx;
-    turtle_y = ty;
-
-    draw_turtle();
-    wanna_x = turtle_x;
-    wanna_y = turtle_y;
-    out_of_bounds = FALSE;
-    pen_state = pen_vis;
-    pen_vis = -1;
-    save_vis();
-    forward_helper((FLONUM)0.0);    /* Lets fill work -- dunno why */
-    pen_vis = pen_state;
-    save_vis();
-
-    done_drawing;
+	done_drawing;
     }
     return(UNBOUND);
 }
@@ -1145,6 +1260,8 @@ void save_line(void) {
 
 void save_move(void) {
     if (safe_to_save()) {
+	if (record_index >= Three && record[record_index - Three] == MOVEXY)
+	    record_index -= Three;
 	record[record_index] = MOVEXY;
 	save_lm_helper();
     }
@@ -1210,6 +1327,22 @@ void save_string(char *s, int len) {
     }
 }
 
+void save_arc(FLONUM count, FLONUM ang, FLONUM radius, FLONUM delta,
+	      FLONUM tx, FLONUM ty, FLONUM angle, FLONUM thead) {
+    if (safe_to_save()) {
+	record[record_index] = ARC;
+	*(FLONUM *)(record + record_index + Big) = count;
+	*(FLONUM *)(record + record_index + 2 * Big) = ang;
+	*(FLONUM *)(record + record_index + 3 * Big) = radius;
+	*(FLONUM *)(record + record_index + 4 * Big) = delta;
+	*(FLONUM *)(record + record_index + 5 * Big) = tx;
+	*(FLONUM *)(record + record_index + 6 * Big) = ty;
+	*(FLONUM *)(record + record_index + 7 * Big) = angle;
+	*(FLONUM *)(record + record_index + 8 * Big) = thead;
+	record_index += 9*Big;
+    }
+}
+
 NODE *lrefresh(NODE *args) {
     refresh_p = TRUE;
     return(UNBOUND);
@@ -1221,19 +1354,24 @@ NODE *lnorefresh(NODE *args) {
 }
 
 void redraw_graphics(void) {
-    int r_index = 0;
+    FLONUM save_tx, save_ty, save_th;
+    FIXNUM r_index = 0;
+    int lastx, lasty;
     pen_info saved_pen;
     BOOLEAN saved_shown;
 #if defined(__ZTC__) && !defined(WIN32)
     BOOLEAN save_splitscreen = in_splitscreen;
 #endif
-    
+   
     if (!refresh_p) {
     /*	clear_screen;
 	draw_turtle();	*/
 	return;
     }
 
+    save_tx = turtle_x;
+    save_ty = turtle_y;
+    save_th = turtle_heading;
     saved_shown = turtle_shown;
     turtle_shown = FALSE;
     save_pen(&saved_pen);
@@ -1244,25 +1382,38 @@ void redraw_graphics(void) {
 #endif
 
     erase_screen();
+    wanna_x = wanna_y = turtle_x = turtle_y = turtle_heading = 0.0;
+    out_of_bounds = FALSE;
+    move_to(screen_x_coord, screen_y_coord);
+    internal_penmode = PENMODE_PAINT;
+    pen_down;
+    set_pen_color((FIXNUM)7);
+    set_pen_width(1);
+    set_pen_height(1);
 
 #ifdef __TURBOC__
     moveto(p_info_x(orig_pen),p_info_y(orig_pen));
 #endif
 
-    while (r_index < record_index)
+    while (r_index < record_index) {
+    	turtle_x = (FLONUM)(lastx-screen_x_center);
+	turtle_y = (FLONUM)(screen_y_center-lasty);
 	switch (record[r_index]) {
 	    case (LINEXY) :
-		line_to(*(int *)(record + r_index + One),
-			*(int *)(record + r_index + Two));
+		lastx = *(int *)(record + r_index + One);
+		lasty = *(int *)(record + r_index + Two);
+		line_to(lastx, lasty);
 		r_index += Three;
 		break;
 	    case (MOVEXY) :
-		move_to(*(int *)(record + r_index + One),
-			*(int *)(record + r_index + Two));
+		lastx = *(int *)(record + r_index + One);
+		lasty = *(int *)(record + r_index + Two);
+		move_to(lastx, lasty);
 		r_index += Three;
 		break;
 	    case (LABEL) :
 		draw_string(record + r_index + One+1);
+		move_to(lastx, lasty);
 		r_index += (One+2 + record[r_index + One] + (One-1)) & ~(One-1);
 		break;
 	    case (SETPENVIS) :
@@ -1275,11 +1426,11 @@ void redraw_graphics(void) {
 #else
 		set_pen_mode(*(int *)(record + r_index + One));
 #endif
-		internal_penmode = *(int *)(record + record_index + Two);
+		internal_penmode = *(int *)(record + r_index + Two);
 		r_index += Three;
 		break;
 	    case (SETPENCOLOR) :
-		set_pen_color(*(int *)(record + r_index + One));
+		set_pen_color((FIXNUM)(*(int *)(record + r_index + One)));
 		r_index += Two;
 		break;
 	    case (SETPENSIZE) :
@@ -1295,7 +1446,20 @@ void redraw_graphics(void) {
 		logofill();
 		r_index += One;
 		break;
+	    case (ARC) :
+		do_arc(*(FLONUM *)(record + r_index + Big),
+		       *(FLONUM *)(record + r_index + 2 * Big),
+		       *(FLONUM *)(record + r_index + 3 * Big),
+		       *(FLONUM *)(record + r_index + 4 * Big),
+		       *(FLONUM *)(record + r_index + 5 * Big),
+		       *(FLONUM *)(record + r_index + 6 * Big),
+		       *(FLONUM *)(record + r_index + 7 * Big),
+		       *(FLONUM *)(record + r_index + 8 * Big),
+		       1);
+		r_index += 9*Big;
+		break;
 	}
+    }
 
     restore_pen(&saved_pen);
     turtle_shown = saved_shown;
@@ -1304,13 +1468,16 @@ void redraw_graphics(void) {
     if (save_splitscreen) {split_screen;}
 #endif
 
+    turtle_x = save_tx;
+    turtle_y = save_ty;
+    turtle_heading = save_th;
     draw_turtle();
 }
 
 /* This is called when the graphics coordinate system has been shifted.
    It adds a constant amount to each x and y coordinate in the record. */
 void resize_record(int dh, int dv) {
-    int r_index = 0;
+    FIXNUM r_index = 0;
     
     p_info_x(orig_pen) += dh;
     p_info_y(orig_pen) += dv;
@@ -1340,5 +1507,209 @@ void resize_record(int dh, int dv) {
 	    case (SETPENPATTERN) :
 		r_index += One+8;
 		break;
+	    case (ARC) :
+		r_index += 9*Big;
+		break;
 	}
 }
+
+NODE *lsavepict(NODE *args) {
+    FILE *fp;
+    static int bg;
+	FIXNUM want, cnt;
+	char *p;
+	
+#if defined(WIN32)||defined(ibm)
+	extern NODE *lopen(NODE *, char *);
+	lopen(args,"wb");
+#else
+    lopenwrite(args);
+#endif
+    if (NOT_THROWING) {
+	fp = (FILE *)file_list->n_obj;
+	save_palette(fp);
+	fwrite(&record_index, sizeof(FIXNUM), 1, fp);
+	want = record_index;
+	p = record;
+	while (want > 0) {
+		cnt = fwrite(p, 1, want, fp);
+		if (ferror(fp) || cnt <= 0) {
+			err_logo(FILE_ERROR,
+				make_static_strnode("File too big"));
+			lclose(args);
+			return UNBOUND;
+		}
+		want -= cnt;
+		p += cnt;
+	}
+	bg = (int)back_ground;
+	fwrite(&bg, sizeof(int), 1, fp);
+	lclose(args);
+	return UNBOUND;
+    }
+}
+
+NODE *lloadpict(NODE *args) {
+    FILE *fp;
+    static int bg;
+	FIXNUM want, cnt;
+	char *p;
+
+#if defined(WIN32)||defined(ibm)
+	extern NODE *lopen(NODE *, char *);
+	lopen(args,"rb");
+#else
+    lopenread(args);
+#endif
+    if (NOT_THROWING) {
+	fp = (FILE *)file_list->n_obj;
+	restore_palette(fp);
+	fread(&record_index, sizeof(FIXNUM), 1, fp);
+	if (record_index < 0 || record_index >= GR_SIZE) {
+		err_logo(FILE_ERROR,
+				 make_static_strnode("File bad format"));
+		record_index = 0;
+		lclose(args);
+		return UNBOUND;
+	}
+	want = record_index;
+	p = record;
+	while (want > 0) {
+		cnt = fread(p, 1, want, fp);
+		if (ferror(fp) || cnt <= 0) {
+			record_index = 0;
+			err_logo(FILE_ERROR,
+					 make_static_strnode("File bad format"));
+			lclose(args);
+			return UNBOUND;
+		}
+		want -= cnt;
+		p += cnt;
+	}
+	fread(&bg, sizeof(int), 1, fp);
+	lclose(args);
+	set_back_ground((FIXNUM)bg);
+	return UNBOUND;
+    }
+}
+
+void ps_string(FILE *fp, char *p) {
+    int ch;
+
+    while ((ch = *p++)) {
+	if (ch=='(' || ch==')' || ch=='\\') fprintf(fp, "\\");
+	fprintf(fp, "%c", ch);
+    }
+}
+
+void rgbprint(FILE *fp, int cnum) {
+    unsigned int r=0, g=0, b=0;
+
+    get_palette(cnum, &r, &g, &b);
+    fprintf(fp, "%6.4f %6.4f %6.4f", ((double)r)/65535,
+		((double)g)/65535, ((double)b)/65535);
+}
+
+#ifdef mac
+extern void fixMacType(NODE *args);
+#endif
+
+NODE *lepspict(NODE *args) {
+    FILE *fp;
+    int r_index = 0, act=0, lastx = 0, lasty = 0, vis = 0;
+
+#ifdef mac
+    fixMacType(args);
+    lopenappend(args);
+#else
+    lopenwrite(args);
+#endif
+    if (NOT_THROWING) {
+	fp = (FILE *)file_list->n_obj;
+
+	fprintf(fp, "%%!PS-Adobe-3.0 EPSF-3.0\n");
+	ndprintf(fp, "%%Title: %p\n", car(args));
+	fprintf(fp, "%%%%Creator: Logo2PS 1.1; Copyright 1997, Vladimir Batagelj\n");
+	fprintf(fp, "%%%%BoundingBox: %d %d %d %d\n",
+		screen_left, screen_top, screen_right, screen_bottom);
+	fprintf(fp, "%%%%EndComments\n");
+	ndprintf(fp, "%%Page: %p\n", car(args));
+	fprintf(fp, "1 setlinecap 1 setlinejoin\n");
+	fprintf(fp, "/Courier 9 selectfont\n");
+
+	fprintf(fp, "gsave\n");
+	rgbprint(fp, back_ground);
+	fprintf(fp, " setrgbcolor\n");
+	fprintf(fp, "%d %d moveto %d %d lineto %d %d lineto %d %d lineto\n",
+		    screen_left, screen_top, screen_right, screen_top,
+		    screen_right, screen_bottom, screen_left, screen_bottom);
+	fprintf(fp, " closepath fill grestore\n");
+
+	fprintf(fp, "%d %d moveto\n", screen_x_center, screen_y_center);
+
+	while (r_index < record_index)
+	    switch (record[r_index]) {
+		case (LINEXY) :
+		    if (!vis) {
+			lastx = *(int *)(record + r_index + One);
+			lasty = screen_height - *(int *)(record + r_index + Two);
+			fprintf(fp, "%d %d lineto\n", lastx, lasty);
+			r_index += Three;
+			act++;
+			break;	/* else fall through */
+		    }
+		case (MOVEXY) :
+		    lastx = *(int *)(record + r_index + One);
+		    lasty = screen_height - *(int *)(record + r_index + Two);
+		    fprintf(fp, "%d %d moveto\n", lastx, lasty);
+		    r_index += Three;
+		    break;
+		case (LABEL) :
+		    fprintf(fp, "gsave -5 1 rmoveto (");
+		    ps_string(fp, record + r_index + One+1);
+		    fprintf(fp, ") show grestore\n");
+		    fprintf(fp, "%d %d moveto\n", lastx, lasty);
+		    r_index += (One+2 + record[r_index + One] + (One-1)) & ~(One-1);
+		    break;
+		case (SETPENVIS) :
+		    vis = record[r_index + 1];
+		    r_index += One;
+		    break;
+		case (SETPENMODE) :
+		    r_index += Three;
+		    break;
+		case (SETPENCOLOR) :
+		    if (act) {
+			fprintf(fp, "stroke %d %d moveto\n", lastx, lasty);
+			act = 0;
+		    }
+		    rgbprint(fp, (*(int *)(record + r_index + One)));
+		    fprintf(fp, " setrgbcolor\n");
+		    r_index += Two;
+		    break;
+		case (SETPENSIZE) :
+		    if (act) {
+			fprintf(fp, "stroke %d %d moveto\n", lastx, lasty);
+			act = 0;
+		    }
+		    fprintf(fp, "%d setlinewidth\n",
+			    (*(int *)(record + r_index + One)));
+		    r_index += Three;
+		    break;
+		case (SETPENPATTERN) :
+		    r_index += One+8;
+		    break;
+		case (FILLERUP) :
+		    r_index += One;
+		    break;
+		case (ARC) :
+		    r_index += 9*Big;
+		    break;
+	    }
+	    
+	fprintf(fp, "stroke\nshowpage\n%%%%EOF\n");
+
+	lclose(args);
+	return UNBOUND;
+    }
+}
