@@ -112,10 +112,12 @@ PRIMTYPE prims[] = {
     {"dribble", 1, 1, 1, PREFIX_PRIORITY, ldribble},
     {"ed", OK_NO_ARG, 1, 1, PREFIX_PRIORITY, ledit},
     {"edit", OK_NO_ARG, 1, 1, PREFIX_PRIORITY, ledit},
+    {"editfile", 1, 1, 1, PREFIX_PRIORITY, leditfile},
     {"emptyp", 1, 1, 1, PREFIX_PRIORITY, lemptyp},
     {"empty?", 1, 1, 1, PREFIX_PRIORITY, lemptyp},
     {"eofp", 0, 0, 0, PREFIX_PRIORITY, leofp},
     {"eof?", 0, 0, 0, PREFIX_PRIORITY, leofp},
+    {"epspict", 1, 1, 1, PREFIX_PRIORITY, lepspict},
     {"equalp", 2, 2, 2, PREFIX_PRIORITY, lequalp},
     {"equal?", 2, 2, 2, PREFIX_PRIORITY, lequalp},
     {"er", 1, 1, 1, PREFIX_PRIORITY, lerase},
@@ -170,6 +172,7 @@ PRIMTYPE prims[] = {
     {"list?", 1, 1, 1, PREFIX_PRIORITY, llistp},
     {"ln", 1, 1, 1, PREFIX_PRIORITY, lln},
     {"load", 1, 1, 1, PREFIX_PRIORITY, lload},
+    {"loadpict", 1, 1, 1, PREFIX_PRIORITY, lloadpict},
     {"local", 1, 1, -1, PREFIX_PRIORITY, llocal},
     {"log10", 1, 1, 1, PREFIX_PRIORITY, llog10},
     {"lowercase", 1, 1, 1, PREFIX_PRIORITY, llowercase},
@@ -265,6 +268,7 @@ PRIMTYPE prims[] = {
     {"runresult", 1, 1, 1, MACRO_PRIORITY, lrunresult},
     {"rw", 0, 0, 0, PREFIX_PRIORITY, lreadword},
     {"save", 1, 1, 1, PREFIX_PRIORITY, lsave},
+    {"savepict", 1, 1, 1, PREFIX_PRIORITY, lsavepict},
     {"scrunch", 0, 0, 0, PREFIX_PRIORITY, lscrunch},
     {"se", 0, 2, -1, PREFIX_PRIORITY, lsentence},
     {"sentence", 0, 2, -1, PREFIX_PRIORITY, lsentence},
@@ -284,7 +288,7 @@ PRIMTYPE prims[] = {
     {"setread", 1, 1, 1, PREFIX_PRIORITY, lsetread},
     {"setreadpos", 1, 1, 1, PREFIX_PRIORITY, lsetreadpos},
     {"setscrunch", 2, 2, 2, PREFIX_PRIORITY, lsetscrunch},
-#ifdef WIN32
+#if defined(WIN32)|defined(ibm)
     {"settc", 2, 2, 2, PREFIX_PRIORITY, set_text_color},
     {"settextcolor", 2, 2, 2, PREFIX_PRIORITY, set_text_color},
 #endif
@@ -305,6 +309,7 @@ PRIMTYPE prims[] = {
     {"st", 0, 0, 0, PREFIX_PRIORITY, lshowturtle},
     {"standout", 1, 1, 1, PREFIX_PRIORITY, lstandout},
     {"step", 1, 1, 1, PREFIX_PRIORITY, lstep},
+    {"stepped", 0, 0, 0, PREFIX_PRIORITY, lstepped},
     {"stop", 0, 0, 0, STOP_PRIORITY, lstop},
     {"substringp", 2, 2, 2, PREFIX_PRIORITY, lsubstringp},
     {"substring?", 2, 2, 2, PREFIX_PRIORITY, lsubstringp},
@@ -319,6 +324,7 @@ PRIMTYPE prims[] = {
     {"tone", 2, 2, 2, PREFIX_PRIORITY, ltone},
     {"towards", 1, 1, 1, PREFIX_PRIORITY, ltowards},
     {"trace", 1, 1, 1, PREFIX_PRIORITY, ltrace},
+    {"traced", 0, 0, 0, PREFIX_PRIORITY, ltraced},
     {"ts", 0, 0, 0, PREFIX_PRIORITY, ltextscreen},
     {"type", 0, 1, -1, PREFIX_PRIORITY, ltype},
     {"unbury", 1, 1, 1, PREFIX_PRIORITY, lunbury},
