@@ -34,7 +34,7 @@
 #define GCMAX 8000
 #else
 #ifdef __ZTC__
-#define GCMAX 5800
+#define GCMAX 4000
 #else
 #define GCMAX 16000
 
@@ -327,8 +327,8 @@ void gc_inc () {
 				(gc_stack_size + GCMAX))) == NULL) {
 
 	    /* no room to increse GC Stack */
-	    printf ("\nWarning: Not enough memory to run garbage collector.\n");
-	    printf ("GC disabled - Save important data and exit!\n");
+	    ndprintf(stdout,"\nWarning: Not enough memory to run garbage collector.\n");
+	    ndprintf(stdout,"GC disabled - Save important data and exit!\n");
 
 	    gc_overflow_flag = 1;
 	} else {
